//#######################################################################
//#
//#	Copyright 	Earth People Technology Inc. 2015
//#
//#
//# File Name:  EPT_4CE6_AF_D1_Top.v
//# Author:     Earth People Technology
//# Date:       March 25, 2019
//# Revision:   A
//#
//# Development: EPT Platform Demonstration Project 
//# Application: Altera Cyclone IV FPGA
//# Description: This file contains verilog code which will allow access
//#              to Active Transfer Library. 
//#               
//#              
//#
//#************************************************************************
//#
//# Revision History:	
//#			DATE		VERSION		DETAILS		
//#			3/25/19 	1			Created
//#
//#									
//#
//#######################################################################


//************************************************************************
//* Module Declaration
//************************************************************************

module EPT_4CE6_AF_D1_Top (

	
	input  wire [1:0]          aa,
	input  wire [1:0]          bc_in,
	output wire [2:0]          bc_out,
	inout  wire [7:0]		   bd_inout,
	
	input  wire   [7:0]        XIO_1,       //XIO -- UB
	input  wire   [7:0]        XIO_2,       //XIO -- UB
	input  wire   [5:0]        XIO_3,       //XIO -- UB
	input  wire   [5:0]        XIO_4,       //XIO -- UB
	output wire   [5:0]        XIO_5,       //XIO -- UB
	output wire   [31:0]       XIO_6,       //XIO -- UB
	output wire   [3:0]        XIO_7       //XIO -- UB
	
	
	);

   //-----------------------------------------------
   // Parameters
   //-----------------------------------------------
   
   parameter                   GLOBAL_RESET_COUNT = 12'h09c8;
   
   //Transfer to Host State Machine
   parameter                   TRANSFER_HOST_IDLE = 1,
                               TRANSFER_HOST_LOOPBACK_START = 2,
							   TRANSFER_HOST_LOOPBACK_COMPLETE = 3;

    // Block Transfer Options
	parameter                  BLOCK_COUNT_16       = 8'h10;

	//Block Transfer to Host State Machine
   parameter                   BLOCK_LOOPBACK_IDLE = 1,
                               BLOCK_LOOPBACK_INIT = 2,
							   FIFO_EN_HIGH = 3,
							   FIFO_EN_LOW = 4;

	//Block Transfer to Host Counter
   parameter                   BLK_TRANSFER_OUT_IDLE = 1,
			                   BLK_TRANSFER_OUT_COUNT = 2;

   //LED State Machine Parameters
   parameter                   IDLE                  = 0,
                               LOAD_IMAGE            = 1,
	                           SELECT_MODE           = 2,
							   UPDATE_IMAGE          = 3,
							   WAIT_FOR_TIMER_HIGH_Z = 4,
                               LOAD_LED_ARRAY        = 5,
                               WAIT_FOR_TIMER_ACTIVE = 6,
                               CHECK_FOR_NEW_IMAGE   = 7;
							   
  //Block In for Image Transfer In
   parameter                   BLOCK_IMAGE_IDLE           = 0,
                               BLOCK_IMAGE_WAIT_FOR_READY = 1,
                               BLOCK_IMAGE_STORE_BYTE     = 2,
                               BLOCK_IMAGE_CHECK_COMPLETE = 3; 

   //Control Register selections for the LED Blinky
   parameter                   RANDOM_NUMBER         = 4'h1;
   parameter                   SHIFT_LEFT            = 4'h2;
   parameter                   SHIFT_RIGHT           = 4'h3;
   parameter                   SCROLL_UP             = 4'h4;
   parameter                   SCROLL_DOWN           = 4'h5;
   parameter                   STATIC_VALUE          = 4'h6;
   parameter                   STATIC_IMAGE          = 4'h7;
   parameter                   BLINK_IMAGE           = 4'h8;
   parameter                   SHIFT_IMAGE_RIGHT     = 4'h9;
   parameter                   SHIFT_IMAGE_LEFT      = 4'ha;  
   parameter                   SHIFT_IMAGE_DOWN      = 4'hb;
   parameter                   SHIFT_IMAGE_UP        = 4'hc;

   //Timer Value Low Limit
`ifdef SIM
   parameter                   TIMER_LOW_LIMIT       = 24'h1f;
   parameter                   HIGH_Z_TIMER_VALUE    = 24'h8f;
`else
   parameter                   TIMER_LOW_LIMIT       = 24'h325aa0;
   parameter                   HIGH_Z_TIMER_VALUE    = 24'ha1220; //10 ms
   //parameter                   HIGH_Z_TIMER_VALUE    = 24'h325aa0; //50 ms
`endif

`ifdef SIM
   	reg [8*30:1] state_name;
`endif
   
//***************************************************************************
//* Internal Signals and Registers Declarations
//***************************************************************************
    wire                       CLK_66;
	wire                       RST;

	wire [23:0]                UC_IN;
	wire [21:0]                UC_OUT;

	//Finite State Machine control registers
	reg  [7:0]                 state, next;

	//Trigger Signals
	reg  [7:0]                 trigger_out;
	wire [7:0]                 trigger_in_byte;	
	reg  [7:0]                 trigger_from_host_latch;
	
	//LED registers
	wire                        led_reset;
	
	//Switch registers
	wire                        switch_reset;
	
	//Transfer registers
	reg                        transfer_out_reg;
	wire                       transfer_in_received;
	wire [7:0]                 transfer_in_byte;
	reg  [7:0]                 transfer_out_byte;
	reg  [3:0]                 transfer_to_host_counter;
	reg  [3:0]                 transfer_to_host_state;

    //Transfer Control registers
	wire                       transfer_in_loop_back;
	reg                        transfer_in_received_reg;
	reg   [3:0]                transfer_control_state;
	reg   [7:0]                transfer_control_byte;
	wire  [7:0]                control_register;
	
	//Block Transfer Out registers
	reg  [2:0]                 state_block_loopback;
	reg                        block_out_reg;
	wire                       block_out_received;
	reg  [7:0]                 block_out_counter;
	reg                        start_block_transfer;
	reg                        start_block_transfer_reg;
	wire                       block_byte_ready;
	wire [7:0]                 block_out_byte;
	reg  [1:0]                 block_transfer_out_state;
	
	//Block Transfer In registers
	wire [7:0]                 block_in_data;
	wire                       block_in_rcv;
	wire                       block_in_loopback;
	wire  [7:0]                ept_length;
	wire  [7:0]                uc_length;
	wire                       block_busy;
	reg   [7:0]                block_in_data_counter;
	reg                        fifo_wren_reg;

	//Reset signals
	wire                       reset;
	reg  [11:0]                reset_counter;
	reg                        reset_signal_reg;
	
	//LED Blinky Transfer registers
	wire [35:0]                LED;
	reg                        led_start_transfer;
	reg                        led_start_transfer_reg;
	wire                       led_transfer_in_received;
	reg   [7:0]                led_host_transfer_byte;
	wire  [7:0]                led_device_transfer_byte;

	//LED BLinky Signals
	reg   [23:0]               timer_value;	
	reg   [7:0]                seed_value;
	reg   [23:0]               led_delay_counter;
	reg   [3:0]                random_number;
	reg   [3:0]                random_num_counter;
	wire                       random_number_generated;
	reg                        start_blinky;
	reg   [3:0]                static_load_value;	
	reg   [3:0]                shift_count_value;
	
	//Image buffers
	reg   [215:0]              led_reg;
	wire  [35:0]               led_array;
	reg   [215:0]              active_image_array;
	reg                        idle_leds;
	reg                        load_leds;
	wire                       new_image_ready;
	wire                       exit_blinky;
	wire                       load_ept_image;
	wire                       load_face_1_image;
	
	//Image Files
	wire  [215:0]              ept_logo_image_array; 
	reg                        img_ept_logo_en;
	wire  [215:0]              face_1_image_array;
	reg                        img_face_1_en;
	wire  [215:0]              block_image_array;
	reg                        img_block_en;
	
	//Block Transfer for Image
	reg                        block_out_image_start;
	wire                       block_in_image_rcv;
	wire                       block_byte_image_ready;
	wire                       block_image_busy;
	wire  [7:0]                ept_length_image;
	wire  [7:0]                uc_length_image;
	reg   [7:0]                block_out_byte_image;
	wire  [7:0]                block_in_data_image;
	reg   [7:0]                block_in_image_counter;
	reg   [7:0]                block_in_image_buffer [31:0];	
	reg   [2:0]                state_block_image;
	wire                       load_block_image;
	
	//Registers for 
	
	
	
	
	
	//Block Memory Registers
    reg   [7:0]                 mem_array [256:0];
	reg   [2:0]                 mem_array_index;
	
	//Test Signals


//***************************************************************************
//* 	Signal Assignments	
//***************************************************************************
   
   //Clock and Reset
   assign            CLK_66 = aa[1];
   assign            RST = reset;
   assign            reset = reset_signal_reg;
    
    //Control Register Map
   assign            transfer_in_loop_back = control_register[0];
   assign            load_block_image      = control_register[1];
   assign            load_ept_image        = control_register[2];
   assign            load_face_1_image     = control_register[3];
   
   //Block to host byte
   assign            block_out_byte = mem_array[block_out_counter];
   assign            uc_length = 8'h8;
   
   //New image has been transferred into memory
   assign            new_image_ready = 1'b0;
   
   //Terminate the Blinky code
   assign            exit_blinky = 1'b0;
   
   //LED Mapping  

  // Top Row of LEDs   
  assign            XIO_6[0] = LED[0]; //UB22
  assign            XIO_6[1] = LED[1]; //UB23
  assign            XIO_6[2] = LED[2]; //UB24
  assign            XIO_6[3] = LED[3]; //UB25
  assign            XIO_6[4] = LED[4]; //UB26
  assign            XIO_6[5] = LED[5]; //UB27
  
  // Second from Top Row of LEDs   
  assign            XIO_6[6] = LED[6]; //UB28
  assign            XIO_6[7] = LED[7]; //UB29
  assign            XIO_6[8] = LED[8]; //UB30
  assign            XIO_6[9] = LED[9]; //UB31
  assign            XIO_6[10] = LED[10]; //UB32 
  assign            XIO_6[11] = LED[11]; //UB33

  // Third from Top Row of LEDs   
  assign            XIO_6[12] = LED[12]; //UB34
  assign            XIO_6[13] = LED[13]; //UB35
  assign            XIO_6[14] = LED[14]; //UB36 Cannot use pin 91 as output
  assign            XIO_6[15] = LED[15]; //UB37 Cannot use pin 90 as output
  assign            XIO_6[16] = LED[16]; //UB38 Cannot use pin 89 as output
  assign            XIO_6[17] = LED[17]; //UB39

  // Fourth from Top Row of LEDs   
  assign            XIO_6[18] = LED[18]; //UB40
  assign            XIO_6[19] = LED[19]; //UB41
  assign            XIO_6[20] = LED[20]; //UB42
  assign            XIO_6[21] = LED[21]; //UB43
  assign            XIO_6[22] = LED[22]; //UB44
  assign            XIO_6[23] = LED[23]; //UB45

  // Fifth from Top Row of LEDs   
  assign            XIO_6[24] = LED[24]; //UB46
  assign            XIO_6[25] = LED[25]; //UB47
  assign            XIO_6[26] = LED[26]; //UB48
  assign            XIO_6[27] = LED[27]; //UB49
  assign            XIO_6[28] = LED[28]; //UB50
  assign            XIO_6[29] = LED[29]; //UB51

  // Sixth from Top Row of LEDs   
  assign            XIO_6[30] = LED[30]; //UB52
  assign            XIO_6[31] = LED[31]; //UB53
  assign            XIO_7[0] = LED[32]; //UB68
  assign            XIO_7[1] = LED[33]; //UB69
  assign            XIO_7[2] = LED[34]; //UB70
  assign            XIO_7[3] = LED[35]; //UB71
  
  
//**************************************************
//* 	Reset Signal	
//**************************************************
	always @(posedge CLK_66 or negedge aa[0])
	begin
	  if(!aa[0])
	  begin
			reset_signal_reg <= 1'b0;
			reset_counter <= 0;
	  end
	  else
	  begin
		if( reset_counter < GLOBAL_RESET_COUNT )
		begin
			reset_signal_reg <= 1'b0;
			reset_counter <= reset_counter + 1'b1;
		end
		else
		begin
			reset_signal_reg <= 1'b1;
		end
	  end
	end

   //-----------------------------------------------
   // Trigger Latch latches the trigger byte coming
   // from the host.
   //-----------------------------------------------

  always @(posedge CLK_66 or negedge RST)
  begin
     if(!RST)
	     trigger_from_host_latch = 0;
	else
	 begin
	     if(trigger_in_byte)
		    trigger_from_host_latch <= trigger_in_byte;
	 end
  
  end

   //-----------------------------------------------
   // Transfer to Host in Loop Back Mode 
   //-----------------------------------------------
    always @(posedge CLK_66 or negedge reset) 
	begin
	     if (!reset)
		 begin
			 transfer_out_reg <= 1'b0;
			 transfer_to_host_counter <= 0;
			 transfer_to_host_state <= TRANSFER_HOST_IDLE;
	     end 
		 else 
		 begin
			     transfer_out_reg <= 1'b0;
			case(transfer_to_host_state)
             TRANSFER_HOST_IDLE:
			     if(transfer_in_received & transfer_in_loop_back)	
                 begin			 
			         transfer_out_byte = transfer_in_byte;
			         transfer_to_host_state <= TRANSFER_HOST_LOOPBACK_START;
			     end
             TRANSFER_HOST_LOOPBACK_START:
		         if(transfer_to_host_counter < 4'hf)
			     begin
		             transfer_to_host_counter <= transfer_to_host_counter + 1'd1;
			         transfer_to_host_state <= TRANSFER_HOST_LOOPBACK_START;
			     end
				 else
				 begin
			         transfer_out_reg <= 1'b1;
			         transfer_to_host_state <= TRANSFER_HOST_LOOPBACK_COMPLETE;				 
				 end
             TRANSFER_HOST_LOOPBACK_COMPLETE:
			     begin
			         transfer_out_reg <= 1'b0;
					 transfer_to_host_counter <= 0;
			         transfer_to_host_state <= TRANSFER_HOST_IDLE;
				 end
			 default:
			      begin
			          transfer_out_reg <= 1'b0;
			          transfer_to_host_counter <= 0;
			          transfer_to_host_state <= TRANSFER_HOST_IDLE;
				  end
			 endcase
	     end
    end

   //-----------------------------------------------
   // Block In write to memory array is driven by block_byte_ready
   //-----------------------------------------------
  always @(posedge CLK_66 or negedge RST)
  begin
    if(!RST)
	begin
		 fifo_wren_reg <= 1'b0;
		 block_in_data_counter <= 0;
	end
	else 
	begin
	     if(block_in_rcv & block_byte_ready & !fifo_wren_reg)
		 begin
		     fifo_wren_reg <= 1'b1;
		 end
	     else if(block_in_rcv & block_byte_ready & fifo_wren_reg)
		 begin
		     fifo_wren_reg <= 1'b1;
			 mem_array[block_in_data_counter] <= block_in_data;
		 end
	     else if(block_in_rcv & !block_byte_ready & fifo_wren_reg)
		 begin
		     fifo_wren_reg <= 1'b0;
			 block_in_data_counter <= block_in_data_counter + 1'd1;
		 end
	     else if(!block_in_rcv)
		 begin
		     fifo_wren_reg <= 1'b0;
		     block_in_data_counter <= 0;
		 end
	end 
  end
   

   //-----------------------------------------------
   // Detect Block Transfer Out start
   //-----------------------------------------------
  always @(block_out_reg or trigger_in_byte or RST)
  begin
     if(!RST)
	     start_block_transfer = 1'b0;
	 else if(block_out_reg)
	     start_block_transfer = 1'b0;
	 else if((trigger_in_byte[7]))
	     start_block_transfer = 1'b1;
  
  end
  
   //-----------------------------------------------
   // Registers to start Block Transfer Out
   //-----------------------------------------------
  always @(posedge CLK_66 or negedge RST)
  begin
    if(!RST)
	begin
		 block_out_reg <= 1'b0;
		 start_block_transfer_reg <= 1'b0;
		 block_out_counter <= 0;
		 block_transfer_out_state <= BLK_TRANSFER_OUT_IDLE;
	end
	else
	begin
	     case(block_transfer_out_state)
		     BLK_TRANSFER_OUT_IDLE:
	             if(start_block_transfer )
				 begin
		             start_block_transfer_reg <= 1'b1;
		             block_transfer_out_state <= BLK_TRANSFER_OUT_COUNT;
				 end
			 BLK_TRANSFER_OUT_COUNT:
		         begin
			          block_out_reg <= 1'b1;
	                 if(block_byte_ready)
		             begin
		                 block_out_counter <= block_out_counter + 1'd1;
		             end
		             else if(block_out_counter >= ept_length )
		             begin
		                 block_out_counter <= 0;
			             block_out_reg <= 1'b0;
		                 block_transfer_out_state <= BLK_TRANSFER_OUT_IDLE;
		             end
				 end
			 default:
			      begin
		                 block_out_counter <= 0;
			             block_out_reg <= 1'b0;
		                 block_transfer_out_state <= BLK_TRANSFER_OUT_IDLE;
				  end
			 endcase
	end
  end

   //-----------------------------------------------
   // Block In read fifo enable is driven by block_byte_ready
   //-----------------------------------------------
  always @(posedge CLK_66 or negedge RST)
  begin
    if(!RST)
	begin
		state_block_loopback <= BLOCK_LOOPBACK_IDLE;
	end
	else 
	begin
	     if(block_in_loopback) 
		 begin
		     case(state_block_loopback)
			 BLOCK_LOOPBACK_IDLE:
			 begin
			     if(block_out_reg)
				    state_block_loopback <= BLOCK_LOOPBACK_INIT;
			 end
			 BLOCK_LOOPBACK_INIT:
			 begin
				 state_block_loopback <= FIFO_EN_HIGH;
		     end
			 FIFO_EN_HIGH:
		     begin
	             if(block_byte_ready )
				 begin
				     state_block_loopback <= FIFO_EN_LOW;
				 end
				 else
				 begin
				     state_block_loopback <= FIFO_EN_HIGH;
				 end
		     end
			 FIFO_EN_LOW:
		     begin
				 if(block_out_counter == 0)
				 begin
					 if(block_out_reg)
				         state_block_loopback <= BLOCK_LOOPBACK_IDLE;
				 end
	             else if(!block_byte_ready )
				 begin
				     state_block_loopback <= FIFO_EN_HIGH;
				 end
			 end
			 endcase
		 end
		 else
		 begin
		     state_block_loopback <= BLOCK_LOOPBACK_IDLE;
		 end
	end 
  end

   //-----------------------------------------------
   // Load Image into local buffer
   //-----------------------------------------------

  always @(posedge CLK_66 or negedge RST)
  begin
    if(!RST)
	begin
	     img_block_en <= 1'b0;
	     img_ept_logo_en <= 1'b0;
		 img_face_1_en <= 1'b0;
		 active_image_array <= 0;
    end
	else 
	begin
	   if(load_block_image)
	   begin
	      active_image_array <= block_image_array;
          img_block_en <= 1'b1;
	   end
	   else
	   begin
	      img_block_en <= 1'b0;
	   end

       if(load_ept_image)
	   begin
	      active_image_array <= ept_logo_image_array;
          img_ept_logo_en <= 1'b1;
	   end
	   else
	   begin
	      img_ept_logo_en <= 1'b0;
	   end

       if(load_face_1_image)
	   begin
	      active_image_array <= face_1_image_array;
          img_face_1_en <= 1'b1;
	   end
	   else
	   begin
	      img_face_1_en <= 1'b0;
	   end


	   end 
  end

   //-----------------------------------------------
   // Block In write to Image buffer driven by block_byte_ready_image
   //-----------------------------------------------
  always @(posedge CLK_66 or negedge RST)
  begin
    if(!RST)
	begin
		 state_block_image <= BLOCK_IMAGE_IDLE;
		 block_in_image_counter <= 0;
	end
	else 
	begin
       case(state_block_image)
       BLOCK_IMAGE_IDLE:
       begin
	     if(block_in_image_rcv)
		 begin
			 state_block_image <= BLOCK_IMAGE_WAIT_FOR_READY;
		 end
       end
       BLOCK_IMAGE_WAIT_FOR_READY:
       begin
	     if(block_in_image_rcv & block_byte_image_ready)
		 begin
			 state_block_image <= BLOCK_IMAGE_STORE_BYTE;
		 end
	     else if(!block_in_image_rcv)
		 begin
		     block_in_image_counter <= 0;
			 state_block_image <= BLOCK_IMAGE_IDLE;
		 end
       end
	   BLOCK_IMAGE_STORE_BYTE:
       begin
	     if(block_in_image_rcv & !block_byte_image_ready)
		 begin
			 block_in_image_buffer[block_in_image_counter] <= block_in_data_image;
			 state_block_image <= BLOCK_IMAGE_CHECK_COMPLETE;
		 end
	     else if(!block_in_image_rcv)
		 begin
		     block_in_image_counter <= 0;
			 state_block_image <= BLOCK_IMAGE_IDLE;
		 end
       end
       BLOCK_IMAGE_CHECK_COMPLETE:
       begin
		 block_in_image_counter <= block_in_image_counter + 1'd1;

		 if(block_in_image_rcv & block_byte_image_ready)
		 begin
			 state_block_image <= BLOCK_IMAGE_STORE_BYTE;
		 end
	     else if(block_in_image_rcv)
		 begin
		     //block_in_image_counter <= 0;
			 state_block_image <= BLOCK_IMAGE_WAIT_FOR_READY;
		 end
	     else if(!block_in_image_rcv)
		 begin
		     block_in_image_counter <= 0;
			 state_block_image <= BLOCK_IMAGE_IDLE;
		 end
		end
       endcase
	end 
  end
   


   //-----------------------------------------------
   // Load Image buffer into led_reg
   //-----------------------------------------------
/*
  always @(posedge CLK_66 or negedge RST)
  begin
    if(!RST)
	     static_load_value <= 0;
	else 
	begin
	     if(state[LOAD_IMAGE])
	         led_reg <= active_image_array;
	end 
  end
*/

   //-----------------------------------------------
   // Load Timer Value
   //-----------------------------------------------

  always @(posedge CLK_66 or negedge RST)
  begin
    if(!RST)
	     timer_value <= TIMER_LOW_LIMIT;
	else 
	begin
	     if(trigger_in_byte[4])
	         timer_value[15:8] <= led_device_transfer_byte;
	     else if(trigger_in_byte[5])
	         timer_value[23:16] <= led_device_transfer_byte;
	end 
  end

   //-----------------------------------------------
   // LED Blinky start 
   //-----------------------------------------------
  always @(posedge CLK_66 or negedge RST)
  begin
    if(!RST)
         start_blinky <= 1'b0;
	else 
	begin
	    if(control_register[7:4] > 0)
            start_blinky <= 1'b1;
		else
            start_blinky <= 1'b0;		
 	end 
  end
 
   //-----------------------------------------------
   // LED Delay Timer Counter 
   //-----------------------------------------------
  always @(posedge CLK_66 or negedge RST)
  begin
    if(!RST)
	     led_delay_counter <= TIMER_LOW_LIMIT;
	else 
	begin
	     if(state[SELECT_MODE])
		    led_delay_counter <= HIGH_Z_TIMER_VALUE;
		 else if(state[WAIT_FOR_TIMER_HIGH_Z])
	        led_delay_counter <= led_delay_counter - 1'd1;
		 else if(state[LOAD_LED_ARRAY])
	        led_delay_counter <= timer_value;
		 else if(state[WAIT_FOR_TIMER_ACTIVE])
	        led_delay_counter <= led_delay_counter - 1'd1;
	end 
  end

   //-----------------------------------------------
   // Random Number array
   //-----------------------------------------------
  always @(posedge CLK_66 or negedge RST)
  begin
     if(!RST)
	 begin
	     random_number <= 6'h0;
		 random_num_counter <= 0;
	 end
	else 
	begin
	    if(random_num_counter < 6'h4 )
	    begin
	        random_num_counter <= random_num_counter + 1'd1;
			random_number[random_num_counter] <= random_number_generated;
		end
	    else 
		begin
	        random_num_counter <= 0;
			random_number[random_num_counter] <= random_number_generated;
		end
	end
  end

   //-----------------------------------------------
   // Send status of LEDs to Host
   //-----------------------------------------------
/*  always @(posedge CLK_66 or negedge RST)
  begin
    if(!RST)
	begin
		led_start_transfer <= 1'b0;
        led_host_transfer_byte <= 0;
        led_start_transfer_reg <= 1'b0;
	end
	else 
	begin
        if((state[WAIT_FOR_TIMER] | led_reset) & !led_start_transfer_reg)
		begin
		   led_start_transfer_reg <= 1'b1;
		   if(timer_value > 24'hbb8)
		   begin
		      led_start_transfer <= 1'b1;
		   end
		   if(led_reset)
                led_host_transfer_byte <= {8'h0};
           else
		       led_host_transfer_byte <= {2'h0,LED[5:0]};
		   end
        else if((state[WAIT_FOR_TIMER] | led_reset) & led_start_transfer_reg)
		begin
		   led_start_transfer_reg <= 1'b1;
		   led_start_transfer <= 1'b0;
		   if(led_reset)
                led_host_transfer_byte <= {8'h0};
           else
		       led_host_transfer_byte <= {2'h0,LED[5:0]};
		end
        else if((state[SELECT_MODE] | led_reset) & led_start_transfer_reg)
		begin
		   led_start_transfer_reg <= 1'b0;
		   led_start_transfer <= 1'b0;
           led_host_transfer_byte <= 0;
		end
	end
  end
 */  
   //-----------------------------------------------
   // Select the LED Function from the control
   // register bits 7 to 4 
   // The led_reg has five rows of 20 columns
   //-----------------------------------------------
  always @(posedge CLK_66 or negedge RST)
  begin
    if(!RST)
	     led_reg <= 0;
	else 
	begin
	     if(state[LOAD_IMAGE])
	         led_reg <= active_image_array;
	
        if (state[SELECT_MODE])
	    begin
	         case(control_register[7:4])
		       RANDOM_NUMBER:
		          led_reg <= random_number;
		       SHIFT_IMAGE_RIGHT:
			   begin
		          led_reg[23:1] <= led_reg[22:0];    // Top Row of LEDs
				  led_reg[0] <= led_reg[23];
				  
		          led_reg[47:25] <= led_reg[46:24];  // Second from Top Row of LEDs 
				  led_reg[24] <= led_reg[47];

		          led_reg[71:49] <= led_reg[70:48];  // Third from Top Row of LEDs
				  led_reg[48] <= led_reg[71];

		          led_reg[95:73] <= led_reg[94:72];  // Fourth from Top Row of LEDs
				  led_reg[72] <= led_reg[95];

		          led_reg[119:97] <= led_reg[118:96];  // Fifth from Top Row of LEDs
				  led_reg[96] <= led_reg[119];

		          led_reg[143:121] <= led_reg[142:120];  // Sixth from Top Row of LEDs
				  led_reg[120] <= led_reg[143];

		          led_reg[167:145] <= led_reg[166:144];  // Sevenh from Top Row of LEDs
				  led_reg[144] <= led_reg[167];

		          led_reg[191:169] <= led_reg[190:168];  // Eighth from Top Row of LEDs
				  led_reg[168] <= led_reg[191];

		          led_reg[215:193] <= led_reg[214:192];  // Ninth from Top Row of LEDs
				  led_reg[192] <= led_reg[215];

			   end
		       SHIFT_IMAGE_LEFT:
			   begin
		          led_reg[22:0] <= led_reg[23:1];    // Top Row of LEDs
				  led_reg[23] <= led_reg[0];
				  
		          led_reg[46:24] <= led_reg[47:25];  // Second from Top Row of LEDs 
				  led_reg[47] <= led_reg[24];

		          led_reg[70:48] <= led_reg[71:49];  // Third from Top Row of LEDs
				  led_reg[71] <= led_reg[48];

		          led_reg[94:72] <= led_reg[95:73];  // Fourth from Top Row of LEDs
				  led_reg[95] <= led_reg[72];

		          led_reg[118:96] <= led_reg[119:97];  // Fifth from Top Row of LEDs
				  led_reg[119] <= led_reg[96];

		          led_reg[142:120] <= led_reg[143:121];  // Sixth from Top Row of LEDs
				  led_reg[143] <= led_reg[120];

		          led_reg[166:144] <= led_reg[167:145];  // Seventh from Top Row of LEDs
				  led_reg[167] <= led_reg[144];

		          led_reg[190:168] <= led_reg[191:169];  // Eighth from Top Row of LEDs
				  led_reg[191] <= led_reg[168];

		          led_reg[214:192] <= led_reg[215:121];  // Ninth from Top Row of LEDs
				  led_reg[215] <= led_reg[192];

			   end
		       SCROLL_UP:
			   begin
		          led_reg[23:0] <= led_reg[47:24];    // Top Row of LEDs

		          led_reg[47:24] <= led_reg[71:48];  // Second from Top Row of LEDs 
				  				  
		          led_reg[71:48] <= led_reg[95:72];  // Third from Top Row of LEDs

		          led_reg[95:72] <= led_reg[119:96];  // Fourth from Top Row of LEDs

		          led_reg[119:96] <= led_reg[143:120];  // Fifth from Top Row of LEDs

		          led_reg[143:120] <= led_reg[167:144];  // Sixth from Top Row of LEDs

		          led_reg[167:144] <= led_reg[191:168];  // Seventh from Top Row of LEDs

		          led_reg[191:168] <= led_reg[215:192];  // Eighth from Top Row of LEDs

		          led_reg[215:192] <= led_reg[23:0];  // Ninth from Top Row of LEDs
				  end
		       SCROLL_DOWN:
			   begin
		          led_reg[23:0] <= led_reg[215:192];    // Top Row of LEDs

		          led_reg[47:24] <= led_reg[23:0];  // Second from Top Row of LEDs 
				  				  
		          led_reg[71:48] <= led_reg[47:24];  // Third from Top Row of LEDs

		          led_reg[95:72] <= led_reg[71:48];  // Fourth from Top Row of LEDs

		          led_reg[119:96] <= led_reg[95:72];  // Fifth from Top Row of LEDs

		          led_reg[143:120] <= led_reg[119:96];  // Sixth from Top Row of LEDs

		          led_reg[167:144] <= led_reg[143:120];  // Seventh from Top Row of LEDs

		          led_reg[191:168] <= led_reg[167:144];  // Eighth from Top Row of LEDs

		          led_reg[215:192] <= led_reg[191:168];  // Ninth from Top Row of LEDs
				  end
		       STATIC_VALUE:
		          led_reg <= active_image_array;
		       STATIC_IMAGE:
		          led_reg <= active_image_array;
		       BLINK_IMAGE:
		          led_reg <= active_image_array;
		       SHIFT_RIGHT:
		          led_reg <= active_image_array;
		       SHIFT_LEFT:
		          led_reg <= active_image_array;
		       SHIFT_IMAGE_DOWN:
		          led_reg <= active_image_array;
		       SHIFT_IMAGE_UP:
		          led_reg <= active_image_array;
		       default:
		          led_reg <= 143'h0;
		      endcase 
        end	
    end
  end
  
   //-----------------------------------------------
   // Increment the Shift/Count value from the control
   // register bits 7 to 4   
   //-----------------------------------------------
/*  always @(posedge CLK_66 or negedge RST)
  begin
    if(!RST)
	     shift_count_value <= 0;
	else 
	begin
	    if(trigger_in_byte[1])
		     shift_count_value <= led_device_transfer_byte[5:0];
		else if (state[LOAD_REGISTER])
	    begin
	         case(control_register[7:4])
		     SHIFT_LEFT:
			 begin
		        shift_count_value[0] <= led_reg[5];
		        shift_count_value[5] <= led_reg[4];
		        shift_count_value[4] <= led_reg[3];
		        shift_count_value[3] <= led_reg[2];
		        shift_count_value[2] <= led_reg[1];
		        shift_count_value[1] <= led_reg[0];
		     end
		     SHIFT_RIGHT:
			 begin
		        shift_count_value[0] <= led_reg[1];
		        shift_count_value[5] <= led_reg[0];
		        shift_count_value[4] <= led_reg[5];
		        shift_count_value[3] <= led_reg[4];
		        shift_count_value[2] <= led_reg[3];
		        shift_count_value[1] <= led_reg[2];
		     end
		     COUNT_UP:
		        shift_count_value <= led_reg + 1'd1;
		     COUNT_DOWN:
		        shift_count_value <= led_reg - 1'd1;
		     default:
	            shift_count_value <= 0;
		    endcase   
         end
	end 
  end
*/

   //-----------------------------------------------
   // Set the LED outputs 
   //-----------------------------------------------
  always @(posedge CLK_66 or negedge RST)
  begin
    if(!RST)
	begin
	     idle_leds <= 1'b0;
		 load_leds <= 1'b0;
	end
	else 
	begin
	   if(state[WAIT_FOR_TIMER_HIGH_Z])
	      idle_leds <= 1'b1;
	   else
	      idle_leds <= 1'b0; 
	
	   if(state[LOAD_LED_ARRAY])
	      load_leds <= 1'b1;
	   else
	      load_leds <= 1'b0; 
	end
  end

     //-----------------------------------------------
   // State Machine: Send Pattern to LEDs 
   //-----------------------------------------------
   // Next State Logic
   always @(posedge CLK_66 or negedge RST)
     begin
	if (!RST)
	  begin
	     state <= 8'h00;
	     state[IDLE] <= 1'b1;
	  end
	else
	  state <= next;
     end

     // State Definitions
   always @ ( state or start_blinky or random_number or
		      led_reg or led_delay_counter or control_register or
			  timer_value or shift_count_value) 
     begin
	     next = 8'h00;

	if (state[IDLE])
	  begin
	     if (start_blinky)
	       next[LOAD_IMAGE] = 1'b1;
	     else
	       next[IDLE] = 1'b1;
	  end
	
	if (state[LOAD_IMAGE])
	  begin
	     if (start_blinky)
	       next[SELECT_MODE] = 1'b1;
	     else
	       next[LOAD_IMAGE] = 1'b1;
	  end
	
	if (state[SELECT_MODE])
	begin
       if(control_register[7:4] == 4'h0)        
            next[IDLE] = 1'b1;
         else		 
           next[UPDATE_IMAGE] = 1'b1;
	end

	if (state[UPDATE_IMAGE])
	begin
        next[WAIT_FOR_TIMER_HIGH_Z] = 1'b1;
    end
		
	if (state[WAIT_FOR_TIMER_HIGH_Z])
	begin
	    if(led_delay_counter == 0)
           next[LOAD_LED_ARRAY] = 1'b1;
		else
		  next[WAIT_FOR_TIMER_HIGH_Z] = 1'b1; 
    end
	
	if (state[LOAD_LED_ARRAY])
		if (led_delay_counter <= timer_value)
			next[LOAD_LED_ARRAY] = 1'b1;
		else 
			next[WAIT_FOR_TIMER_ACTIVE] = 1'b1;

	if (state[WAIT_FOR_TIMER_ACTIVE])
	begin
	    if(led_delay_counter == 0)
           next[CHECK_FOR_NEW_IMAGE] = 1'b1;
		else
           next[WAIT_FOR_TIMER_ACTIVE] = 1'b1;
    end

	if (state[CHECK_FOR_NEW_IMAGE])
	begin
		if (new_image_ready)
			next[LOAD_IMAGE] = 1'b1;
		else if(exit_blinky)
			next[IDLE] = 1'b1;
		else 
			next[SELECT_MODE] = 1'b1;
    end

			
`ifdef SIM
 	   if ( state == ( 1 << IDLE ))
		   state_name = "IDLE";
	   else if ( state == ( 1 << LOAD_IMAGE ))
		   state_name = "LOAD_IMAGE";
	   else if ( state == ( 1 << SELECT_MODE ))
		   state_name = "SELECT_MODE";
	   else if ( state == ( 1 << UPDATE_IMAGE ))
		   state_name = "UPDATE_IMAGE";
	   else if ( state == ( 1 << WAIT_FOR_TIMER_HIGH_Z ))
		   state_name = "WAIT_FOR_TIMER_HIGH_Z";
	   else if ( state == ( 1 << LOAD_LED_ARRAY ))
		   state_name = "LOAD_LED_ARRAY";
	   else if ( state == ( 1 << WAIT_FOR_TIMER_ACTIVE ))
		   state_name = "WAIT_FOR_TIMER_ACTIVE";
	   else if ( state == ( 1 << CHECK_FOR_NEW_IMAGE ))
		   state_name = "CHECK_FOR_NEW_IMAGE";
`endif


	end//end of state machine

   //-----------------------------------------------
   // Image Files
   //-----------------------------------------------
  image_block_in               IMG_BLOCK_IN_INST
  (
   .IMAGE_ARRAY                (block_image_array),
   .INPUT_BUFFER               (block_in_data_image),
   .INPUT_BUFFER_INDEX         (block_in_image_counter),
   .ENABLE                     (block_in_image_rcv)

   );	
   
  image_ept_logo               IMG_EPT_LOGO_INST
  (
   .IMAGE_ARRAY                (ept_logo_image_array),
   .ENABLE                     (img_ept_logo_en)

   );	
   
  image_face_1                 IMG_FACE_1_INST
  (
   .IMAGE_ARRAY                (face_1_image_array),
   .ENABLE                     (img_face_1_en)

   );	
   
   //-----------------------------------------------
   // Update LED array
   //-----------------------------------------------
   led_update                  LED_UPDATE_INST
   (
	.CLK                        (CLK_66),
	.RST                        (RST),

	.LED_REG                    (led_reg[143:0]),
	.LED                        (LED),

	.LOAD_LEDS                  (load_leds),
	.IDLE_LEDS                  (idle_leds)


	);


   
   //-----------------------------------------------
   // Linear Feedback Shift Register
   //-----------------------------------------------
  lfsr                         LFSR_INST
  (
   .Q                          (random_number_generated),
   .CLK                        (CLK_66),
   .RST                        (RST),
   .SEED                       (led_device_transfer_byte[3:0]),
   .LOAD                       (trigger_in_byte[6])
   );	
	

   //-----------------------------------------------
   // State Machine: Control Register from Transfer In 
   //-----------------------------------------------
  /*
     Control Register Register Map
	 
	 control_register[0]  =  transfer_in_loop_back
	 control_register[1]  =  load_block_image//block_in_loopback 
	 control_register[2]  =  load_ept_image
	 control_register[3]  =  load_face_1_image
	 control_register[4]  =  LED Select Mode[0]
	 control_register[5]  =  LED Select Mode[1]
	 control_register[6]  =  LED Select Mode[2]
	 control_register[7]  =  LED Select Mode[3]
	 

  */
  active_control_register      ACTIVE_CONTROL_REG_INST
  (
   .CLK                        (CLK_66),
   .RST                        (RST),
   .TRANSFER_IN_RECEIVED       (transfer_in_received),
   .TRANSFER_IN_BYTE           (transfer_in_byte),

   .CONTROL_REGISTER           (control_register)
   );
   
   //-----------------------------------------------
   // Instantiate the EPT Library
   //-----------------------------------------------

	active_transfer_library	   ACTIVE_TRANSFER_LIBRARY_INST
	(	
	.aa                        (aa),
	.bc_in                     (bc_in),
	.bc_out                    (bc_out),
	.bd_inout                  (bd_inout),

	.UC_IN                     (UC_IN),
	.UC_OUT                    (UC_OUT)
	
	);
	
   //-----------------------------------------------
   // Instantiate the EPT Library
   //-----------------------------------------------
wire [22*5-1:0]  uc_out_m;
eptWireOR # (.N(5)) wireOR (UC_OUT, uc_out_m);

/*
    Active Trigger Register Mapping
	
	trigger_in_byte[0]  =  Load Static Value
	trigger_in_byte[1]  =  Load shift_count_value
	trigger_in_byte[2]  =  
	trigger_in_byte[3]  =  
	trigger_in_byte[4]  =  Load Timer Value, timer_value[15:8]
	trigger_in_byte[5]  =  Load Timer Value, timer_value[23:16]
	trigger_in_byte[6]  =  Load linear_feedback_shift_register
	trigger_in_byte[7]  =  start_block_transfer = 1'b1
	

*/
	active_trigger             ACTIVE_TRIGGER_INST
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 0*22 +: 22 ]),

	 .trigger_to_host          (trigger_out),
	 .trigger_to_device        (trigger_in_byte)
	
	);

    active_transfer            ACTIVE_TRANSFER_INST_1
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 1*22 +: 22 ]),
	
	 .start_transfer           (led_start_transfer),
	 .transfer_received        (led_transfer_in_received),
	 
	 .transfer_busy            (),
	
	 .uc_addr                  (3'h1),

	 .transfer_to_host         (led_host_transfer_byte),
	 .transfer_to_device       (led_device_transfer_byte)	
    );
	
	active_transfer            ACTIVE_TRANSFER_INST_2
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 2*22 +: 22 ]),
	
	 .start_transfer           (transfer_out_reg),
	 .transfer_received        (transfer_in_received),
	
	 .transfer_busy            (),
	
	 .uc_addr                  (3'h2),

	 .transfer_to_host         (transfer_out_byte),
	 .transfer_to_device       (transfer_in_byte)	
	);
	
	
	active_block               BLOCK_TRANSFER_INST
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 3*22 +: 22 ]),
	
	 .start_transfer           (block_out_reg),
	 .transfer_received        (block_in_rcv),
	 
	 .transfer_ready           (block_byte_ready),
	 .transfer_busy            (block_busy),

	 .ept_length               (ept_length),
	
	 .uc_addr                  (3'h4),
	 .uc_length                (ept_length),

	 .transfer_to_host         (block_out_byte),
	 .transfer_to_device       (block_in_data)
	
	);

	active_block               BLOCK_IMAGE_INST
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 4*22 +: 22 ]),
	
	 .start_transfer           (block_out_image_start),
	 .transfer_received        (block_in_image_rcv),
	 
	 .transfer_ready           (block_byte_image_ready),
	 .transfer_busy            (block_image_busy),

	 .ept_length               (ept_length_image),
	
	 .uc_addr                  (3'h2),
	 .uc_length                (ept_length_image),

	 .transfer_to_host         (block_out_byte_image),
	 .transfer_to_device       (block_in_data_image)
	
	);

	
endmodule

